﻿//******************************************************************************
// MIDIシーケンサーソフト『世界樹』
// トラックリストモードスケールビュークラス
// (C)2002-2017 おーぷんMIDIぷろじぇくと／くず
//******************************************************************************

/* This Source Code Form is subject to the terms of the Mozilla Public */
/* License, v. 2.0. If a copy of the MPL was not distributed with this */
/* file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "stdafx.h"
#include "resource.h"
#include "..\\..\\MIDIIO\\MIDIIO.h"
#include "..\\..\\MIDIData\\MIDIData.h"
#include "..\\..\\MIDIClock\\MIDIClock.h"
#include "..\\..\\MIDIStatus\\MIDIStatus.h"
#include "..\\..\\MIDIInstrument\\MIDIInstrument.h"
#include "HistoryRecord.h"
#include "HistoryUnit.h"
#include "SekaijuApp.h"
#include "SekaijuDoc.h"
#include "SekaijuView.h"
#include "SekaijuToolbar.h"
#include "SekaijuStatusBar.h"
#include "ChildFrame.h"
#include "TrackListFrame.h"
#include "TrackListModeScaleView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

IMPLEMENT_DYNCREATE (CTrackListModeScaleView, CSekaijuView)

// メッセージマップ
BEGIN_MESSAGE_MAP (CTrackListModeScaleView, CSekaijuView)
	ON_WM_CREATE ()
	ON_WM_KEYDOWN ()	
	ON_WM_LBUTTONDOWN ()	
	ON_WM_RBUTTONDOWN ()	
	ON_WM_LBUTTONUP ()	
	ON_WM_RBUTTONUP ()	
	ON_WM_MOUSEMOVE ()
	ON_WM_MOUSEWHEEL ()
END_MESSAGE_MAP ()

//------------------------------------------------------------------------------
// 構築と破壊
//------------------------------------------------------------------------------

// コンストラクタ
CTrackListModeScaleView::CTrackListModeScaleView () {
}

// デストラクタ
CTrackListModeScaleView::~CTrackListModeScaleView () {
}

//------------------------------------------------------------------------------
// オーバーライド
//------------------------------------------------------------------------------

// 原点の移動
void CTrackListModeScaleView::OnPrepareDC (CDC* pDC, CPrintInfo* pInfo) {
	CTrackListFrame* pTrackListFrame = (CTrackListFrame*)GetParent ();
	pDC->SetWindowOrg (pTrackListFrame->GetColumnScrollPos (), 0);
}

// 描画
void CTrackListModeScaleView::OnDraw (CDC* pDC) {
	CSekaijuApp* pSekaijuApp = (CSekaijuApp*)AfxGetApp ();
	CTrackListFrame* pTrackListFrame = (CTrackListFrame*)GetParent ();

	// 画面のdpi取得
	HDC hDC = ::GetDC (this->GetSafeHwnd ());
	long lLogPixelsX = ::GetDeviceCaps (hDC, LOGPIXELSX);
	long lLogPixelsY = ::GetDeviceCaps (hDC, LOGPIXELSY);
	::ReleaseDC (this->GetSafeHwnd (), hDC);

	CRect rcClient;
	GetClientRect (&rcClient);
	pDC->DPtoLP (&rcClient);
	CFont* pOldFont = pDC->SelectObject (&(pSekaijuApp->m_theFont[0]));
	CRect theRect (0, 0, 0, 0);
	long lColumnZoom = pTrackListFrame->GetColumnZoom ();
	long lColorBtnFace = ::GetSysColor (COLOR_BTNFACE);
	long lColorBtnShadow = ::GetSysColor (COLOR_BTNSHADOW);
	long lColorBtnHighlight = ::GetSysColor (COLOR_BTNHIGHLIGHT);
	long lColorBtnText = ::GetSysColor (COLOR_BTNTEXT);
	long lColorBlack = RGB (0, 0, 0);
	long lColorWhite = RGB (255, 255, 255);
	// 背景の塗りつぶし
	pDC->FillSolidRect (&rcClient, lColorBtnFace);
	pDC->SetBkMode (TRANSPARENT);
	// 各列の描画
	pDC->SetTextColor (lColorBtnText);
	for (int j = 0; j < TRACKLISTFRAME_NUMCOLUMN; j ++) {
		theRect.top = rcClient.top;
		theRect.bottom = rcClient.bottom;
		theRect.left = theRect.right; 
		theRect.right = theRect.right + pTrackListFrame->GetColumnBaseWidth (j) * lColumnZoom * lLogPixelsX / 96;
		pDC->Draw3dRect (&theRect, lColorBtnHighlight, lColorBtnShadow);
		pDC->DrawText (pTrackListFrame->GetColumnTitle (j), &theRect, DT_LEFT | DT_VCENTER | DT_SINGLELINE);
	}
	pDC->SelectObject (pOldFont);
}

//------------------------------------------------------------------------------
// メッセージマップ
//------------------------------------------------------------------------------

// ウィンドウ生成時
BOOL CTrackListModeScaleView::OnCreate (LPCREATESTRUCT lpcs) {
	return CSekaijuView::OnCreate (lpcs);
}

// キー押し下げ時
void CTrackListModeScaleView::OnKeyDown (UINT nChar, UINT nRepCnt, UINT nFlags) {
	CTrackListFrame* pTrackListFrame = (CTrackListFrame*)GetParent ();
	pTrackListFrame->PostMessage (WM_KEYDOWN, nChar, (nFlags << 16) | nRepCnt);
}


// マウス左ボタン押された時  
void CTrackListModeScaleView::OnLButtonDown (UINT nFlags, CPoint point) {
	CTrackListFrame* pTrackListFrame = (CTrackListFrame*)GetParent ();
	CSekaijuApp* pSekaijuApp = (CSekaijuApp*)AfxGetApp ();
	point += CSize (pTrackListFrame->GetColumnScrollPos (), 0);

	// 画面のdpi取得
	HDC hDC = ::GetDC (this->GetSafeHwnd ());
	long lLogPixelsX = ::GetDeviceCaps (hDC, LOGPIXELSX);
	long lLogPixelsY = ::GetDeviceCaps (hDC, LOGPIXELSY);
	::ReleaseDC (this->GetSafeHwnd (), hDC);

	// カーソルが境界上にあるか調べる
	long j = 0;
	long lBorderX = 0;
	for (j = 0; j < TRACKLISTFRAME_NUMCOLUMN; j++) {
		lBorderX += pTrackListFrame->GetColumnWidth (j, lLogPixelsX);
		if (lBorderX - 2 * lLogPixelsX / 96 <= point.x && point.x <= lBorderX + 2 * lLogPixelsX / 96) {
			break;
		}
	}
	// カーソルが境界上にあった場合
	if (0 <= j && j < TRACKLISTFRAME_NUMCOLUMN) {
		SetCapture ();
		m_lTempColumnIndex = j;
		m_lTempColumnBaseWidth = pTrackListFrame->GetColumnBaseWidth (j);
		m_ptMouseDown = m_ptMouseMoveOld = point;
		::SetCursor (pSekaijuApp->m_hCursorSizeWE);
	}
}

// マウス右ボタン押された時  
void CTrackListModeScaleView::OnRButtonDown (UINT nFlags, CPoint point) {

}

// マウス左ボタン離されたとき
void CTrackListModeScaleView::OnLButtonUp (UINT nFlags, CPoint point) {
	CTrackListFrame* pTrackListFrame = (CTrackListFrame*)GetParent ();
	CSekaijuApp* pSekaijuApp = (CSekaijuApp*)AfxGetApp ();

	// 画面のdpi取得
	HDC hDC = ::GetDC (this->GetSafeHwnd ());
	long lLogPixelsX = ::GetDeviceCaps (hDC, LOGPIXELSX);
	long lLogPixelsY = ::GetDeviceCaps (hDC, LOGPIXELSY);
	::ReleaseDC (this->GetSafeHwnd (), hDC);

	if (GetCapture () == this) {
		ReleaseCapture ();
		::SetCursor (pSekaijuApp->m_hCursorArrow);
		pTrackListFrame->RecalcColumnScrollInfo (lLogPixelsY);
		pTrackListFrame->m_pModeScaleView->Invalidate (lLogPixelsX);
		pTrackListFrame->m_pTrackModeView->Invalidate (lLogPixelsX);
	}
}

// マウス右ボタン離されたとき
void CTrackListModeScaleView::OnRButtonUp (UINT nFlags, CPoint point) {

}

// マウスが動かされたとき
void CTrackListModeScaleView::OnMouseMove (UINT nFlags, CPoint point) {
	CTrackListFrame* pTrackListFrame = (CTrackListFrame*)GetParent ();
	CSekaijuApp* pSekaijuApp = (CSekaijuApp*)AfxGetApp ();
	point += CSize (pTrackListFrame->GetColumnScrollPos (), 0);

	// 画面のdpi取得
	HDC hDC = ::GetDC (this->GetSafeHwnd ());
	long lLogPixelsX = ::GetDeviceCaps (hDC, LOGPIXELSX);
	long lLogPixelsY = ::GetDeviceCaps (hDC, LOGPIXELSY);
	::ReleaseDC (this->GetSafeHwnd (), hDC);

	// キャプター中
	if (GetCapture () == this) {
		CRect rcClient;
		GetClientRect (&rcClient);
		CSize szMouseDelta = point.x - m_ptMouseDown.x;
		long lNewColumnBaseWidth = 
			(m_lTempColumnBaseWidth * pTrackListFrame->GetColumnZoom () + szMouseDelta.cx) / 
			pTrackListFrame->GetColumnZoom ();
		lNewColumnBaseWidth = CLIP (1, lNewColumnBaseWidth, 1024);
		if (lNewColumnBaseWidth != pTrackListFrame->GetColumnBaseWidth (m_lTempColumnIndex)) {
			CTrackListFrame* pTrackListFrame = (CTrackListFrame*)GetParent ();
			pTrackListFrame->SetColumnBaseWidth (m_lTempColumnIndex, lNewColumnBaseWidth);
			pTrackListFrame->m_pModeScaleView->Invalidate ();
			pTrackListFrame->m_pTrackModeView->Invalidate ();
		}
		m_ptMouseMoveOld = point;
	}
	// 非キャプター中
	else {
		// カーソルが境界上にあるか調べる
		long j = 0;
		long lBorderX = 0;
		for (j = 0; j < TRACKLISTFRAME_NUMCOLUMN; j++) {
			lBorderX += pTrackListFrame->GetColumnWidth (j, lLogPixelsX);
			if (lBorderX - 2 * lLogPixelsX / 96 <= point.x && point.x <= lBorderX + 2 * lLogPixelsX / 96) {
				break;
			}
		}
		// カーソルが境界上にあった場合
		if (0 <= j && j < TRACKLISTFRAME_NUMCOLUMN) {
			::SetCursor (pSekaijuApp->m_hCursorSizeWE);
		}
		// カーソルが境界上にない場合
		else {
			::SetCursor (pSekaijuApp->m_hCursorArrow);
		}
	}
}

// マウスホイールが回された時
BOOL CTrackListModeScaleView::OnMouseWheel (UINT nFlags, short zDelta, CPoint point) {
	CTrackListFrame* pTrackListFrame = (CTrackListFrame*)GetParent ();
	CSekaijuApp* pSekaijuApp = (CSekaijuApp*)AfxGetApp ();
	CSekaijuDoc* pSekaijuDoc = GetDocument ();

	// 画面のdpi取得
	HDC hDC = ::GetDC (this->GetSafeHwnd ());
	long lLogPixelsX = ::GetDeviceCaps (hDC, LOGPIXELSX);
	long lLogPixelsY = ::GetDeviceCaps (hDC, LOGPIXELSY);
	::ReleaseDC (this->GetSafeHwnd (), hDC);

	long lDelta = zDelta;
	long lFlags = nFlags;
	if (lFlags & MK_CONTROL) {
		if (lDelta > 0 && pSekaijuApp->m_theGeneralOption.m_bInvertCtrlMouseWheel == 0 ||
			lDelta < 0 && pSekaijuApp->m_theGeneralOption.m_bInvertCtrlMouseWheel != 0) {
			pTrackListFrame->PostMessage (WM_COMMAND, ID_CONTROL_PREVMEASURE);
		}
		else {
			pTrackListFrame->PostMessage (WM_COMMAND, ID_CONTROL_NEXTMEASURE);
		}
	}
	else {
		long lRowScrollPos = pTrackListFrame->GetRowScrollPos ();
		long lRowZoom = pTrackListFrame->GetRowZoom ();
		lRowScrollPos -= lRowZoom * lLogPixelsY / 96 * lDelta / 120;
		pTrackListFrame->SetRowScrollPos (lRowScrollPos);
	}
	return TRUE;
}
